<?php


namespace EventPortal\Tests\Unit\Util;

use Carbon\Carbon;
use EventPortal\Domain\Util\IOEngage\Exceptions\AuthenticationFailedException;
use EventPortal\Domain\Util\IOEngage\Exceptions\DelegateRetrievalFailed;
use EventPortal\Domain\Util\IOEngage\Exceptions\SearchFailedException;
use EventPortal\Domain\Util\IOEngage\IOEngage;
use EventPortal\Tests\TestCase;
use GuzzleHttp\Client;
use Illuminate\Http\Response;
use Illuminate\Support\Arr;
use Mockery\MockInterface;
use Spatie\Snapshots\MatchesSnapshots;

class IOEngageTest extends TestCase
{
    use MatchesSnapshots;

    /** @test */
    public function can_authenticate(): void
    {
        cache(['ioengage' => null]);

        $this->assertEmpty(cache('ioengage'));
        $this->client()->authenticate();

        $cache = cache('ioengage');
        $this->assertNotEmpty($cache);
        $this->assertArrayHasKey('token', $cache);
        $this->assertArrayHasKey('expires', $cache);
    }

    /** @test */
    public function does_not_re_auhenticate_if_token_not_expired(): void
    {
        cache(['ioengage' => [
            'token' => 'ACCESS_TOKEN',
            'expires' => Carbon::now()->addSeconds(600),
        ]]);

        $this->client()->authenticate();

        $cache = cache('ioengage');
        $this->assertEquals('ACCESS_TOKEN', Arr::get($cache, 'token'));
    }

    /** @test */
    public function throws_exception_if_authentication_payload_is_bad(): void
    {
        $this->markTestIncomplete('Unable mock POST method on \GuzzleHttp\Client');

        $mock = $this->partialMock(Client::class, function ($mock) {
            $mock->shouldNotReceive('post')->andReturn(null);
        });

        $this->expectException(AuthenticationFailed::class);

        $this->client()->authenticate();
    }

    /** @test */
    public function can_make_authenticated_request(): void
    {
        $result = $this->client()->delegates('WUKA00005610', 'WUKF00001918');
        $this->assertMatchesJsonSnapshot((array) $result);
    }

    /** @test */
    public function get_delegates_fails_properly(): void
    {
        $this->expectException(DelegateRetrievalFailed::class);
        $this->client()->delegates('FAKEID');
    }

    /** @test */
    public function can_search_for_delegates(): void
    {
        $response = $this->client()->search(['FirstName' => 'Jeff']);
        $this->assertMatchesJsonSnapshot((array) $response);
    }

    public function can_mark_delegate_as_attending_meeting(): void
    {
        $this->markTestIncomplete('Requires further clarification. Will need new queue set up for bulk updates.');
    }

    private function client(): IOEngage
    {
        return app(IOEngage::class);
    }
}
